@@ -102,6 +102,7 @@ sqlite_libencoding
 sqlite_changes
 sqlite_create_function
 sqlite_create_aggregate
+sqlite_function_type
 sqlite_user_data
 sqlite_aggregate_context
 sqlite_aggregate_count
@@ -112,6 +113,11 @@ sqlite_set_result_error
 sqliteMalloc
 sqliteFree
 sqliteRealloc
+sqlite_set_authorizer
+sqlite_trace
+sqlite_compile
+sqlite_step
+sqlite_finalize
 END_OF_FILE
 i386-mingw32msvc-dllwrap \
      --def sqlite.def -v --export-all \
