@@ -12,7 +12,7 @@
 ** This is the implementation of generic hash-tables
 ** used in SQLite.
 **
-** $Id: hash.c,v 1.8 2002/05/21 23:44:30 drh Exp $
+** $Id: hash.c,v 1.10 2003/05/12 23:06:53 drh Exp $
 */
 #include "sqliteInt.h"
 #include <assert.h>
@@ -108,8 +108,7 @@ static int binHash(const void *pKey, int nKey){
   while( nKey-- > 0 ){
     h = (h<<3) ^ h ^ *(z++);
   }
-  if( h<0 ) h = -h;
-  return h;
+  return h & 0x7fffffff;
 }
 static int binCompare(const void *pKey1, int n1, const void *pKey2, int n2){
   if( n1!=n2 ) return n2-n1;
@@ -313,7 +312,7 @@ void *sqliteHashInsert(Hash *pH, const void *pKey, int nKey, void *data){
   new_elem = (HashElem*)sqliteMalloc( sizeof(HashElem) );
   if( new_elem==0 ) return data;
   if( pH->copyKey && pKey!=0 ){
-    new_elem->pKey = sqliteMalloc( nKey );
+    new_elem->pKey = sqliteMallocRaw( nKey );
     if( new_elem->pKey==0 ){
       sqliteFree(new_elem);
       return data;
