@@ -13,8 +13,9 @@
 ** is not included in the SQLite library.  It is used for automated
 ** testing of the SQLite library.
 **
-** $Id: test2.c,v 1.9 2002/08/12 12:29:57 drh Exp $
+** $Id: test2.c,v 1.15 2003/02/11 14:55:41 drh Exp $
 */
+#include "os.h"
 #include "sqliteInt.h"
 #include "pager.h"
 #include "tcl.h"
@@ -48,6 +49,7 @@ static char *errorName(int rc){
     case SQLITE_CONSTRAINT: zName = "SQLITE_CONSTRAINT";  break;
     case SQLITE_MISMATCH:   zName = "SQLITE_MISMATCH";    break;
     case SQLITE_MISUSE:     zName = "SQLITE_MISUSE";      break;
+    case SQLITE_NOLFS:      zName = "SQLITE_NOLFS";       break;
     default:                zName = "SQLITE_Unknown";     break;
   }
   return zName;
@@ -62,7 +64,7 @@ static int pager_open(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Pager *pPager;
   int nPage;
@@ -74,7 +76,7 @@ static int pager_open(
     return TCL_ERROR;
   }
   if( Tcl_GetInt(interp, argv[2], &nPage) ) return TCL_ERROR;
-  rc = sqlitepager_open(&pPager, argv[1], nPage, 0);
+  rc = sqlitepager_open(&pPager, argv[1], nPage, 0, 1);
   if( rc!=SQLITE_OK ){
     Tcl_AppendResult(interp, errorName(rc), 0);
     return TCL_ERROR;
@@ -93,7 +95,7 @@ static int pager_close(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Pager *pPager;
   int rc;
@@ -120,7 +122,7 @@ static int pager_rollback(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Pager *pPager;
   int rc;
@@ -147,7 +149,7 @@ static int pager_commit(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Pager *pPager;
   int rc;
@@ -174,7 +176,7 @@ static int pager_ckpt_begin(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Pager *pPager;
   int rc;
@@ -201,7 +203,7 @@ static int pager_ckpt_rollback(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Pager *pPager;
   int rc;
@@ -228,7 +230,7 @@ static int pager_ckpt_commit(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Pager *pPager;
   int rc;
@@ -255,7 +257,7 @@ static int pager_stats(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Pager *pPager;
   int i, *a;
@@ -288,7 +290,7 @@ static int pager_pagecount(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Pager *pPager;
   char zBuf[100];
@@ -312,7 +314,7 @@ static int page_get(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Pager *pPager;
   char zBuf[100];
@@ -346,7 +348,7 @@ static int page_lookup(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   Pager *pPager;
   char zBuf[100];
@@ -376,7 +378,7 @@ static int page_unref(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   void *pPage;
   int rc;
@@ -403,7 +405,7 @@ static int page_read(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   char zBuf[100];
   void *pPage;
@@ -427,7 +429,7 @@ static int page_number(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   char zBuf[100];
   void *pPage;
@@ -451,7 +453,7 @@ static int page_write(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
-  char **argv            /* Text of each argument */
+  const char **argv      /* Text of each argument */
 ){
   void *pPage;
   int rc;
@@ -471,31 +473,88 @@ static int page_write(
   return TCL_OK;
 }
 
+/*
+** Usage:   fake_big_file  N  FILENAME
+**
+** Write a few bytes at the N megabyte point of FILENAME.  This will
+** create a large file.  If the file was a valid SQLite database, then
+** the next time the database is opened, SQLite will begin allocating
+** new pages after N.  If N is 2096 or bigger, this will test the
+** ability of SQLite to write to large files.
+*/
+static int fake_big_file(
+  void *NotUsed,
+  Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
+  int argc,              /* Number of arguments */
+  const char **argv      /* Text of each argument */
+){
+  int rc;
+  int n;
+  off_t offset;
+  OsFile fd;
+  int readOnly = 0;
+  if( argc!=3 ){
+    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
+       " N-MEGABYTES FILE\"", 0);
+    return TCL_ERROR;
+  }
+  if( Tcl_GetInt(interp, argv[1], &n) ) return TCL_ERROR;
+  rc = sqliteOsOpenReadWrite(argv[2], &fd, &readOnly);
+  if( rc ){
+    Tcl_AppendResult(interp, "open failed: ", errorName(rc), 0);
+    return TCL_ERROR;
+  }
+  offset = n;
+  offset *= 1024*1024;
+  rc = sqliteOsSeek(&fd, offset);
+  if( rc ){
+    Tcl_AppendResult(interp, "seek failed: ", errorName(rc), 0);
+    return TCL_ERROR;
+  }
+  rc = sqliteOsWrite(&fd, "Hello, World!", 14);
+  sqliteOsClose(&fd);
+  if( rc ){
+    Tcl_AppendResult(interp, "write failed: ", errorName(rc), 0);
+    return TCL_ERROR;
+  }
+  return TCL_OK;
+}
+
 /*
 ** Register commands with the TCL interpreter.
 */
 int Sqlitetest2_Init(Tcl_Interp *interp){
   extern int sqlite_io_error_pending;
-  Tcl_CreateCommand(interp, "pager_open", pager_open, 0, 0);
-  Tcl_CreateCommand(interp, "pager_close", pager_close, 0, 0);
-  Tcl_CreateCommand(interp, "pager_commit", pager_commit, 0, 0);
-  Tcl_CreateCommand(interp, "pager_rollback", pager_rollback, 0, 0);
-  Tcl_CreateCommand(interp, "pager_ckpt_begin", pager_ckpt_begin, 0, 0);
-  Tcl_CreateCommand(interp, "pager_ckpt_commit", pager_ckpt_commit, 0, 0);
-  Tcl_CreateCommand(interp, "pager_ckpt_rollback", pager_ckpt_rollback, 0, 0);
-  Tcl_CreateCommand(interp, "pager_stats", pager_stats, 0, 0);
-  Tcl_CreateCommand(interp, "pager_pagecount", pager_pagecount, 0, 0);
-  Tcl_CreateCommand(interp, "page_get", page_get, 0, 0);
-  Tcl_CreateCommand(interp, "page_lookup", page_lookup, 0, 0);
-  Tcl_CreateCommand(interp, "page_unref", page_unref, 0, 0);
-  Tcl_CreateCommand(interp, "page_read", page_read, 0, 0);
-  Tcl_CreateCommand(interp, "page_write", page_write, 0, 0);
-  Tcl_CreateCommand(interp, "page_number", page_number, 0, 0);
+  static struct {
+    char *zName;
+    Tcl_CmdProc *xProc;
+  } aCmd[] = {
+    { "pager_open",              (Tcl_CmdProc*)pager_open          },
+    { "pager_close",             (Tcl_CmdProc*)pager_close         },
+    { "pager_commit",            (Tcl_CmdProc*)pager_commit        },
+    { "pager_rollback",          (Tcl_CmdProc*)pager_rollback      },
+    { "pager_ckpt_begin",        (Tcl_CmdProc*)pager_ckpt_begin    },
+    { "pager_ckpt_commit",       (Tcl_CmdProc*)pager_ckpt_commit   },
+    { "pager_ckpt_rollback",     (Tcl_CmdProc*)pager_ckpt_rollback },
+    { "pager_stats",             (Tcl_CmdProc*)pager_stats         },
+    { "pager_pagecount",         (Tcl_CmdProc*)pager_pagecount     },
+    { "page_get",                (Tcl_CmdProc*)page_get            },
+    { "page_lookup",             (Tcl_CmdProc*)page_lookup         },
+    { "page_unref",              (Tcl_CmdProc*)page_unref          },
+    { "page_read",               (Tcl_CmdProc*)page_read           },
+    { "page_write",              (Tcl_CmdProc*)page_write          },
+    { "page_number",             (Tcl_CmdProc*)page_number         },
+    { "fake_big_file",           (Tcl_CmdProc*)fake_big_file       },
+  };
+  int i;
+  for(i=0; i<sizeof(aCmd)/sizeof(aCmd[0]); i++){
+    Tcl_CreateCommand(interp, aCmd[i].zName, aCmd[i].xProc, 0, 0);
+  }
   Tcl_LinkVar(interp, "sqlite_io_error_pending",
      (char*)&sqlite_io_error_pending, TCL_LINK_INT);
 #ifdef SQLITE_TEST
-  Tcl_LinkVar(interp, "pager_old_format",
-     (char*)&pager_old_format, TCL_LINK_INT);
+  Tcl_LinkVar(interp, "journal_format",
+     (char*)&journal_format, TCL_LINK_INT);
 #endif
   return TCL_OK;
 }
